/*
 *
 * Copyright(C) OASIS(R) 2010. All Rights Reserved.
 * OASIS trademark, IPR and other policies apply.
 *
 */

/**
 * Test initiation program implementation with a single reference of multiplicity 0..1
 * @author Bryan Aupperle
 *
 */

#include "TestInvocationP.h"
#include "SCA.h"
#include <wchar.h>

/* @Property */
char testName[50];

/* @Reference(name="reference1", interfaceHeader="Service1P.h", required="false") */

main () {
    wchar_t myService [255];
    wchar_t myOperation [255];
    struct invokeTest request;
    struct invokeTestResponse response;
	struct TestException testFault;
	SCAREF serviceToken;
	int compCode, reason, size = 50, num_values = 1;
	char *pTestName = testName, response1[255], faultTemp[25], *result;
	wchar_t faultName[25], *pFaultName = faultName;

	SCAPropertyCString(L"testName", &pTestName, &size, &num_values, &compCode, &reason);

    SCAService(myService, &compCode, &reason);

    SCAOperation(myOperation, &compCode, &reason);

    if (wcscmp(myOperation,L"invokeTest") == 0){
    	SCAMessageIn(myService, myOperation, sizeof(request), (void *)&request,
                     &compCode, &reason);

		result = (char *)malloc(255);
		SCAGetReference(L"reference1", &serviceToken, &compCode, &reason);
		if (serviceToken) {
			size = 255;
			SCAInvoke(serviceToken, L"operation1", strlen(request.input), (void *)request.input,
	                  &size, (void *)response1, &compCode, &reason);

			switch (compCode) {
				case (SCACC_FAULT):
					size = 255;
					SCAGetFaultMessage(serviceToken, &size, &pFaultName, (void *)response1, &compCode, &reason);

					strcpy(result, "Test service got a fault during execution: ");
					wcstombs(faultTemp, faultName, 25);
					strcat(result, faultTemp);
					strcat(result, " - ");
					strcat(result, response1);
					testFault.faultInfo = result;
					SCASetFaultMessage(L"", L"", L"TestException", sizeof(testFault), (void *)&testFault,
					                   &compCode, &reason);

					result[0] = '\0';
					break;

				case (SCACC_ERROR):
					strcpy(result, "Test service got an error during execution: ");
					switch (reason) {
						case (SCA_SERVICE_UNAVAILABLE):
							strcat(result, "Service unavailable");
							break;
						case (SCA_MULTIPLE_SERVICES):
							strcat(result, "Multiple services");
							break;
						case (SCA_PARAMETER_ERROR):
							strcat(result, "Incorrect parameter value");
							break;
						default:
							strcat(result, "Other error");
					}
					testFault.faultInfo = result;
					SCASetFaultMessage(L"", L"", L"TestException", sizeof(testFault), (void *)&testFault,
					                   &compCode, &reason);

					result[0] = '\0';
					break;

				case (SCACC_OK):
					strcpy(result, testName);
					strcat(result, " ");
					strcat(result, request.input);
					strcat(result, " ");
					strcat(result, response1);
					break;

				default:
					/* Should not get here */
					result[0] = '\0';
			}
		}
		else {
			strcpy(result, testName);
			strcat(result, " ");
			strcat(result, request.input);
			strcat(result, "  no invocation");
		}

		response.result = result;
        SCAMessageOut(myService, myOperation, sizeof(response),(void *)&response,
                      &compCode, &reason);
        free(result);
    }
};
